@@ -35,6 +35,8 @@ def create_project_config(project_name: str) -> str:
         metadata:
           team: development
           environment: local
+          service_context: ""
+          # Describe the overall service scenario (used by multi-turn supervisor)
           # Add any custom fields used by dashboards or automation tools.
         """
     ).strip() + "\n"
@@ -120,7 +122,7 @@ def create_simulation_config(project_name: str) -> str:
         name: {project_name}_experiment
         description: AI agent simulation experiment
 
-        iterations: 10           # Number of times to cycle through inputs/personas
+        iterations: 1           # Number of times to cycle through inputs/personas
         parallel_runs: 1          # Increase for concurrent execution (ensure thread safety)
         run_delay_seconds: 0      # Optional delay between runs to avoid rate limits
         seed: 42                  # Set for reproducibility; remove for randomness
@@ -139,6 +141,25 @@ def create_simulation_config(project_name: str) -> str:
           recording_file: recordings/args_recording.jsonl
           override_param_path: data.content
 
+        multi_turn:
+          enabled: false              # Enable to drive conversations via supervisor
+          max_turns: 8                # Safety cap on total turns per conversation
+          auto_approve_tools: true    # Automatically approve tool calls when supported
+          persona_override: null      # Force a specific persona id (optional)
+          supervisor:
+            provider: openai          # openai (LLM generated) | mock (scripted playback)
+            model: gpt-5-mini
+            system_prompt: |
+              You supervise an AI assistant supporting customers.
+              Review the entire transcript and decide whether to continue.
+              When continuing, craft the next user message consistent with the persona.
+              When terminating, explain the reason and provide any closing notes.
+            metadata:
+              scripted_questions: []  # Array of user utterances for sequential playback (e.g., ["First question", "Second question", ...])
+              mock_decision: terminate        # Fallback when no scripted questions remain
+              mock_reason: script_complete    # Termination reason for scripted runs
+              mock_closing: "Thanks for the help. I have no further questions."
+
         output_directory: experiments
         save_traces: true
         save_aggregated_metrics: true
@@ -159,6 +180,12 @@ def create_evaluation_config() -> str:
     clarity_sample = get_prompt_bundle("response_clarity").sample_response
     completeness_sample = get_prompt_bundle("information_completeness").sample_response
 
+    sample_indent = "                "
+    intent_sample_block = indent(intent_sample.strip(), sample_indent)
+    consistency_sample_block = indent(consistency_sample.strip(), sample_indent)
+    clarity_sample_block = indent(clarity_sample.strip(), sample_indent)
+    completeness_sample_block = indent(completeness_sample.strip(), sample_indent)
+
     return dedent(
         f"""
         # FluxLoop Evaluation Configuration
@@ -234,7 +261,8 @@ def create_evaluation_config() -> str:
             max_score: 10
             parser: first_number_1_10
             metadata:
-              sample_response: "{intent_sample}"
+              sample_response: |
+{intent_sample_block}
 
           - name: response_consistency
             type: llm_judge
@@ -251,7 +279,8 @@ def create_evaluation_config() -> str:
             max_score: 10
             parser: first_number_1_10
             metadata:
-              sample_response: "{consistency_sample}"
+              sample_response: |
+{consistency_sample_block}
 
           - name: response_clarity
             type: llm_judge
@@ -268,7 +297,8 @@ def create_evaluation_config() -> str:
             max_score: 10
             parser: first_number_1_10
             metadata:
-              sample_response: "{clarity_sample}"
+              sample_response: |
+{clarity_sample_block}
 
           - name: information_completeness
             type: llm_judge
@@ -285,7 +315,8 @@ def create_evaluation_config() -> str:
             max_score: 10
             parser: first_number_1_10
             metadata:
-              sample_response: "{completeness_sample}"
+              sample_response: |
+{completeness_sample_block}
 
         # ------------------------------------------------------------
         # Aggregation Settings
@@ -297,7 +328,7 @@ def create_evaluation_config() -> str:
         # ------------------------------------------------------------
         # Limits (LLM cost controls)
         limits:
-          sample_rate: 0.3          # evaluate 30% of traces with LLM
+          sample_rate: 1.0          # evaluate 100% of traces with LLM
           max_llm_calls: 50         # cap total LLM API calls
           timeout_seconds: 60
           cache: evaluation_cache.jsonl
